@@ -2,6 +2,7 @@ from Products.ATContentTypes.criteria import path
 
 from collective.formcriteria.criteria import common
 
+
 class FormPathCriterion(
     common.FormCriterion, path.ATPathCriterion):
     __doc__ = path.ATPathCriterion.__doc__
@@ -12,5 +13,8 @@ class FormPathCriterion(
     schema['formFields'].vocabulary = common.makeVocabularyForFields(
         schema['value'], schema['recurse'])
 
+    def Recurse(self, **kw):
+        return self.getFormFieldValue('recurse', **kw)
+
 common.registerCriterion(
     FormPathCriterion, orig=path.ATPathCriterion)
