@@ -1,39 +1,50 @@
 import urllib
 
 from zope.component import getMultiAdapter
-from zope.app import pagetemplate
 from zope.i18n import translate
 
 from Acquisition import aq_inner
+from Missing import MV
+from DateTime.interfaces import IDateTime
+
+from Products.Five.browser import metaconfigure
+from Products.Five.browser import pagetemplatefile
 
 from Products.CMFCore.utils import getToolByName
+from Products.CMFCore import Expression
 
 from Products.CMFPlone.utils import safe_unicode
-from Products.CMFPlone import PloneBatch  
-from plone.memoize import instance
+from Products.CMFPlone import PloneBatch
+from plone.memoize import view
 
 from plone.app.content.browser import tableview
 from plone.app.content.browser import foldercontents
 
-class Table(tableview.Table):
-    """Use a table template which obeys the columns fields"""                
 
-    render = pagetemplate.ViewPageTemplateFile("table.pt")
+class Table(metaconfigure.ViewMixinForTemplates, tableview.Table):
+    """Use a table template which obeys the columns fields"""
 
-    def __init__(self, context, request, base_url, view_url, items,
-                 batch, columns, show_sort_column=False, buttons=[],
-                 pagesize=20):
+    index = pagetemplatefile.ViewPageTemplateFile("table.pt")
+
+    def render(self, *args, **kw):
+        """Delegate to the registered template"""
+        return self.index(*args, **kw)
+
+    def update(self, base_url, view_url, items, batch, columns,
+               show_sort_column=False, buttons=[], pagesize=20):
         self._batch = batch
-        super(Table, self).__init__(
-            request=request, base_url=base_url, view_url=view_url,
-            items=items, show_sort_column=show_sort_column,
-            buttons=buttons, pagesize=batch.size)
+        context = aq_inner(self.context)
+        tableview.Table.__init__(
+            self, request=self.request, base_url=base_url,
+            view_url=view_url, items=items,
+            show_sort_column=show_sort_column, buttons=buttons,
+            pagesize=batch.size)
         map(self.set_checked, items)
-        self.context = context
+        self.__dict__['context'] = context
         self.columns = columns
 
     @property
-    @instance.memoize
+    @view.memoize
     def batch(self):
         """Use the collection batch"""
         return self._batch
@@ -73,8 +84,12 @@ class Table(tableview.Table):
     def within_batch_size(self):
         return self.batch.sequence_length < self.pagesize
 
+
 class FolderContentsTable(foldercontents.FolderContentsTable):
-    """Use a table template which obeys the columns fields"""                
+    """Use a table template which obeys the columns fields"""
+
+    # Used for evaluating TALES
+    index = pagetemplatefile.ViewPageTemplateFile('empty.pt')
 
     # Copied from
     # plone.app.content.browser.foldercontents.FolderContentsTable
@@ -84,24 +99,29 @@ class FolderContentsTable(foldercontents.FolderContentsTable):
         self.context = context
         self.request = request
         self.contentFilter = request
-        
-        column_vocab = context.Vocabulary('customViewFields')[0]
-        link_columns = context.getField(
-            'customViewLinks').getAccessor(context)()
-        columns = [
-            dict(id=column, name=column_vocab.getValue(column),
-                 link=(column in link_columns))
-            for column in context.getCustomViewFields()]
+
+        sort_info = context.restrictedTraverse(
+            '@@sort_info').getSortInfo()
+        self.columns = context.restrictedTraverse('columns_view')
+
+        # If a sort has been selected by the user, make sure it will
+        # be picked up by the topic.  This is done by adding it to the
+        # request.
+        sort_on = contentFilter.get('sort_on')
+        if sort_on in sort_info['sorts']:
+            sort = self.columns.sorts[sort_on]
+            request[sort] = True
 
         url = context.absolute_url()
         view_url = url + '/@@folder_contents'
-        self.table = Table(
-            context, request, url, view_url, self.items, self.batch,
-            columns, show_sort_column=self.show_sort_column,
+        self.table = context.restrictedTraverse(
+            '@@folder_contents_table')
+        self.table.update(url, view_url, self.items, self.batch,
+            self.columns, show_sort_column=self.show_sort_column,
             buttons=self.buttons)
 
     @property
-    @instance.memoize
+    @view.memoize
     def batch(self):
         """Let the collection batch the results"""
         context = aq_inner(self.context)
@@ -114,9 +134,11 @@ class FolderContentsTable(foldercontents.FolderContentsTable):
         return context.queryCatalog(self.contentFilter, batch=True)
 
     @property
-    @instance.memoize
+    @view.memoize
     def items(self):
         """Use the item brains"""
+        # Mostly copied from plone.app.content.browser.foldercontents,
+        # modified to use catalog brains
         context = aq_inner(self.context)
         plone_utils = getToolByName(context, 'plone_utils')
         plone_view = getMultiAdapter((context, self.request), name=u'plone')
@@ -124,10 +146,14 @@ class FolderContentsTable(foldercontents.FolderContentsTable):
         portal_properties = getToolByName(context, 'portal_properties')
         portal_types = getToolByName(context, 'portal_types')
         site_properties = portal_properties.site_properties
-        
-        use_view_action = site_properties.getProperty('typesUseViewActionInListings', ())
+        portal = getToolByName(context, 'portal_url').getPortalObject()
+
+        use_view_action = site_properties.getProperty(
+            'typesUseViewActionInListings', ())
         browser_default = context.browserDefault()
 
+        econtext = Expression.getExprContext(context)
+
         results = []
         for i, obj in enumerate(self.batch):
             if (i + 1) % 2 == 0:
@@ -137,8 +163,8 @@ class FolderContentsTable(foldercontents.FolderContentsTable):
 
             url = obj.getURL()
             path = obj.getPath or "/".join(obj.getPhysicalPath())
-            icon = plone_view.getIcon(obj);
-            
+            icon = plone_view.getIcon(obj)
+
             type_class = 'contenttype-' + plone_utils.normalizeString(
                 obj.portal_type)
 
@@ -158,37 +184,73 @@ class FolderContentsTable(foldercontents.FolderContentsTable):
             if obj_type in use_view_action:
                 view_url = url + '/view'
             elif obj.is_folderish:
-                view_url = url + "/folder_contents"              
+                view_url = url + "/folder_contents"
             else:
                 view_url = url
 
             is_browser_default = len(browser_default[1]) == 1 and (
                 obj.id == browser_default[1][0])
-                                 
+
+            columns = {}
+            for column in self.columns.ordered:
+                value = getattr(obj, column['field'], MV)
+
+                # Calculate the sums before using any expression
+                if value is not MV:
+                    if column['has_sum']:
+                        if 'sum' in column:
+                            column['sum'] += value
+                        else:
+                            column['sum'] = value
+
+                expr = column.get('expr')
+                if expr:
+                    econtext.vars.update(item=obj, value=value)
+                    value = econtext.evaluate(expr)
+                    del econtext.vars['value']
+                    del econtext.vars['item']
+
+                if IDateTime.providedBy(value):
+                    value = plone_view.toLocalizedTime(value, long_format=1)
+
+                columns[column['field']] = value
+
             results.append(dict(
-                url = url,
-                url_href_title = url_href_title,
-                id  = obj.getId,
-                quoted_id = urllib.quote_plus(obj.getId),
-                path = path,
-                title_or_id = obj.pretty_title_or_id(),
-                obj_type = obj_type,
-                size = obj.getObjSize,
-                modified = modified,
-                icon = icon.html_tag(),
-                type_class = type_class,
-                wf_state = review_state,
-                state_title = portal_workflow.getTitleForStateOnType(review_state,
-                                                           obj_type),
-                state_class = state_class,
-                is_browser_default = is_browser_default,
-                folderish = obj.is_folderish,
-                relative_url = relative_url,
-                view_url = view_url,
-                table_row_class = table_row_class,
-                is_expired = context.isExpired(obj),
+                url=url,
+                url_href_title=url_href_title,
+                id=obj.getId,
+                quoted_id=urllib.quote_plus(obj.getId),
+                path=path,
+                title_or_id=obj.pretty_title_or_id(),
+                obj_type=obj_type,
+                size=obj.getObjSize,
+                modified=modified,
+                icon=icon.html_tag(),
+                type_class=type_class,
+                wf_state=review_state,
+                state_title=portal_workflow.getTitleForStateOnType(
+                    review_state, obj_type),
+                state_class=state_class,
+                is_browser_default=is_browser_default,
+                folderish=obj.is_folderish,
+                relative_url=relative_url,
+                view_url=view_url,
+                table_row_class=table_row_class,
+                is_expired=context.isExpired(obj),
                 obj=obj,
-            ))
+                columns=columns,
+                ))
+
+        for column in self.columns.ordered:
+            if not column['has_sum'] or 'sum' not in column:
+                continue
+            expr = column.get('expr')
+            if expr:
+                econtext.vars.update(item=obj, value=column['sum'])
+                column['sum'] = econtext.evaluate(expr)
+                del econtext.vars['value']
+                del econtext.vars['item']
+
         return results
 
     @property
@@ -214,14 +276,38 @@ class FolderContentsTable(foldercontents.FolderContentsTable):
             if button['id'] != 'paste' or context.cb_dataValid():
                 buttons.append(self.setbuttonclass(button))
         return buttons
-    
-class FolderContentsView(foldercontents.FolderContentsView):
+
+
+class FolderContentsMixin(object):
+
+    def __call__(self, *args, **kw):
+        context = aq_inner(self.context)
+        self.columns = context.restrictedTraverse('columns_view')
+        return super(FolderContentsMixin, self).__call__(*args, **kw)
+
+
+class FolderContentsView(FolderContentsMixin,
+                         foldercontents.FolderContentsView):
     """List items in a tabular form including object buttons"""
-    
+
+    def __init__(self, context, request):
+        # Bypass the interface request declaration that blocks the
+        # workflow menu
+        super(foldercontents.FolderContentsView, self).__init__(
+            context, request)
+
     def contents_table(self):
         """Use the request as the contentFilter"""
         table = FolderContentsTable(self.context, self.request)
         return table.render()
 
-class FolderContentsKSSView(foldercontents.FolderContentsKSSView):
+
+if hasattr(foldercontents, 'FolderContentsKSSView'):
+    # BBB Plone 4.2
+    FolderContentsBrowserView = foldercontents.FolderContentsKSSView
+else:
+    FolderContentsBrowserView = foldercontents.FolderContentsBrowserView
+
+
+class FolderContentsKSSView(FolderContentsMixin, FolderContentsBrowserView):
     table = FolderContentsTable
