@@ -1,11 +1,12 @@
 from zope import interface
 
 import AccessControl
+from Products.ZCatalog.Lazy import LazyCat
+from Products.ZCTextIndex.ParseTree import ParseError
 
 from Products.Archetypes import atapi
 from Products.ATContentTypes import interfaces as atct_ifaces
 from Products.ATContentTypes import permission
-from Products.ATContentTypes import config 
 from Products.ATContentTypes.content import topic
 from Products.ATContentTypes.tool import topic as topic_tool
 from Products.ATContentTypes import criteria
@@ -14,6 +15,7 @@ from Products.CMFPlone import CatalogTool
 from collective.formcriteria import interfaces
 from collective.formcriteria.criteria import sort
 
+
 class Topic(topic.ATTopic):
     """A collection supporting form criteria"""
     interface.implements(interfaces.IFormTopic)
@@ -21,31 +23,17 @@ class Topic(topic.ATTopic):
     security = AccessControl.ClassSecurityInfo()
 
     schema = topic.ATTopic.schema.copy() + atapi.Schema((
-        atapi.LinesField(
-            'customViewLinks',
-            default=('Title',),
-            vocabulary='listMetaDataFields',
-            enforceVocabulary=True,
-            write_permission=permission.ChangeTopics,
-            widget=atapi.InAndOutWidget(
-                label=u'Table Column Links',
-                description=
-                u"Select which table columns to link to the item.")),
         atapi.StringField(
             'formLayout',
             default='atct_topic_view',
             vocabulary='getPossibleFormLayouts',
             widget=atapi.SelectionWidget(
                 label=u'Form Results Layout',
-                description=
-                u'Select the display layout use for results.  '
-                u'Used only with the "Search Form" layout')),
+                description=(
+                    u'Select the display layout use for results.  '
+                    u'Used only with the "Search Form" layout'))),
         ))
-    schema['customViewFields'].default = [
-        'Title',
-        'getObjSize',
-        'ModificationDate',
-        'review_state']
+    del schema['customViewFields']
 
     sort_indices = {
         'unsorted': topic_tool.TopicIndex(
@@ -63,7 +51,7 @@ class Topic(topic.ATTopic):
 
     sort_vocab = atapi.DisplayList([(sort.FormSortCriterion.meta_type,
                                      sort.FormSortCriterion.shortDesc)])
-    
+
     def getPossibleFormLayouts(self):
         """Return all valid form results display laouts"""
         return atapi.DisplayList(
@@ -96,7 +84,6 @@ class Topic(topic.ATTopic):
                     return self.sort_vocab
                 return [sort.FormSortCriterion.meta_type]
 
-    security.declareProtected(permission.ChangeTopics, 'getIndex')
     def getIndex(self, name):
         """Get index data including sort indices"""
         tool = self.portal_atct
@@ -106,6 +93,7 @@ class Topic(topic.ATTopic):
             if name in self.sort_indices:
                 return self.sort_indices[name]
             raise
+    security.declareProtected(permission.ChangeTopics, 'getIndex')
 
     def listFields(self):
         """Add sort fields"""
@@ -114,26 +102,75 @@ class Topic(topic.ATTopic):
             (key, value.friendlyName, value.description)
             for key, value in self.sort_indices.items()]
 
+    def listQueryCriteria(self):
+        """Return a list of our query criteria objects.
+        """
+        providedBy = getattr(
+            atct_ifaces.IATTopicSortCriterion, 'providedBy', None)
+        if providedBy is None:
+            # BBB Plone 3
+            providedBy = (
+                atct_ifaces.IATTopicSortCriterion.isImplementedBy)
+        return [val for val in self.listCriteria() if not providedBy(val)]
+
+    def listQueryCriteriaVocab(self):
+        """Return a list of fields for which this topic has query
+        criteria."""
+        return [('', '')] + [
+            (crit.Field(), crit.Title())
+            for crit in self.listQueryCriteria()]
+
     def listSortCriteria(self):
         """Return a list of our sort criteria objects.
         """
-        return [val for val in self.listCriteria() if
-                atct_ifaces.IATTopicSortCriterion.isImplementedBy(val)]
+        providedBy = getattr(
+            atct_ifaces.IATTopicSortCriterion, 'providedBy', None)
+        if providedBy is None:
+            # BBB Plone 3
+            providedBy = (
+                atct_ifaces.IATTopicSortCriterion.isImplementedBy)
+        return [val for val in self.listCriteria() if providedBy(val)]
+
+    def listSortCriteriaVocab(self):
+        """Return a list of fields for which this topic has sort
+        criteria."""
+        return [('', '')] + [
+            (crit.Field(), crit.Title())
+            for crit in self.listSortCriteria()]
 
     def getFriendlyName(self, index):
         """Get the friendly name for an index from the tool"""
-        tool = getattr(self, config.TOOLNAME)
+        tool = getattr(self, 'portal_atct')
         try:
             tool.getIndex(index)
         except AttributeError:
             if index in self.sort_indices:
                 return self.sort_indices[index].friendlyName
-        return tool.getFriendlyName(index)        
+        return tool.getFriendlyName(index)
 
     def addCriterion(self, field, criterion_type):
         """Make sure that criteria are properly initialized."""
         crit = super(Topic, self).addCriterion(field, criterion_type)
         crit.initializeArchetype()
         return crit
-    
+
+    def getCustomViewFields(self):
+        columns = getattr(self, 'columns', [])
+        if columns:
+            return [
+                column.Field() for column in columns.contentValues()]
+        return columns
+
+    def queryCatalog(self, *args, **kw):
+        """
+        Gracefully handle index query parsing errors.
+
+        Such as those raised by text index queries containing '('.
+        """
+        try:
+            return super(Topic, self).queryCatalog(*args, **kw)
+        except ParseError:
+            return LazyCat([[]])
+
+
 atapi.registerType(Topic, 'collective.formcriteria')
